// Title:   BilateralDenoiser.h
// Created: Thu Sep 25 15:11:48 2003
// Authors: Richard Keiser, Mark Pauly, Sascha Scandella
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __BILATERALDENOISER_H_
#define __BILATERALDENOISER_H_

#include "../../../Core/DataStructures/src/NeighbourHood.h"
#include "../../../Core/Scene/src/Scene.h"
#include "../../../Core/Scene/src/Object.h"


/**
 * Bilateral Denoiser for point-sampled geometry. Note that normals
 * are not justified - but for distance field calculation the denoiser
 * is sometimes useful. (Derived from Code of Thouis R. Jones: 
 * Non-Iterative, Feature-Preserving Mesh Smoothing)
 */
class BilateralDenoiser {

public:
	
	/**
	 */	
	BilateralDenoiser() {};
	
	/**
	 * Destructor removes data structures.
	 */	
	~BilateralDenoiser() {	
		delete [] surfelPositions;
		delete [] newSurfelPositions;
		delete [] surfelNormals;
		delete [] newSurfelNormals;
		delete nbHood;
	};

	/**
	 * This method has to be called in order to smooth the
	 * positions of a given point-cloud.
	 * @param sigma_f
	 *			is the spatial sigma.
	 * @param sigma_g
	 *			is the influence sigma
	 */
	void smooth(float sigma_f, float sigma_g);

protected:
	
	// the gaussian filter function
	float gaussianSquared(float r, float sigma);

	// the filter function
	void filter(int curPosIndex, Vector3D *weightedPos, float *k, float spatial_sigma, float influence_sigma, bool mollify);

	// filter vertex
	void filterVertex(int curPosIndex);

	// mollify vertex
	void mollifyVertex(int curPosIndex);


protected:

	/// Spatial search data structure (kd-Tree)
	NeighbourHood *nbHood;
	/// Spatial sigma
	float sigma_f;
	/// Influence sigma
	float sigma_g;
	/// Sigma for mollification
	float sigma_m;
	/// Cutoff value
	float cutoff;
	/// Average length between surfels.
	float meanLength;
	/// Surfel positions
	Vector3D *surfelPositions;
	/// New surfel positions
	Vector3D *newSurfelPositions;
	/// Surfel normals
	Vector3D *surfelNormals;
	/// New surfel normals
	Vector3D *newSurfelNormals;
};

#endif	// __BILATERALDENOISER_H_
// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
